import { AfterViewInit, Component, ElementRef, NgZone, ViewChild } from '@angular/core';
import { UsersService } from './services/users.service';
import { ScrollService } from './services/scroll.service';
import { Message, MessagesService } from './services/messages.service';

@Component({
    selector: 'app-root',
    templateUrl: './app.component.html',
    styleUrls: ['./app.component.scss']
})
export class AppComponent implements AfterViewInit {
    @ViewChild('container') container!: ElementRef<HTMLDivElement>;

    public messages: Message[] = [];
    public removedMessages: Message[] = [];

    public ngAfterViewInit(): void {
        this.scroll.scrollHost = this.container.nativeElement;
    }

    public despawnMsg(id: number) {
        this.zone.run(() => {
            this.removedMessages = this.removedMessages.filter(v => v.id !== id);
        });
    }

    constructor(
        public users: UsersService,
        public scroll: ScrollService,
        public zone: NgZone,
        public _messages: MessagesService,
    ) {
        _messages.added.subscribe(v => {
            this.messages.push(v);
        });
        _messages.removed.subscribe(v => {
            this.messages.splice(this.messages.indexOf(v), 1);
            this.removedMessages.push(v);
        });
    }
}
