import { HttpErrorResponse } from '@angular/common/http';
import { EventEmitter, Injectable } from '@angular/core';

export type Role = 'info' | 'success' | 'warning' | 'error';

export interface Message {
    content: string;
    role: string;
    timestamp: number;
    id: number;
}

@Injectable({
    providedIn: 'root'
})
export class MessagesService {
    private _messages: Message[] = [];

    constructor() { }

    public get messages() {
        return this._messages;
    }

    public added = new EventEmitter<Message>();
    public removed = new EventEmitter<Message>();

    public add(message: string, role: Role) {
        const id = Math.random();
        const msg = {
            content: message,
            role: role,
            timestamp: Date.now(),
            id,
        } as Message;
        this._messages.push(msg);

        if (msg.role === 'error') console.error('ERROR:', msg.content);
        else console.log(msg.role, msg.content);

        const timeout = setTimeout(() => {
            this.removed.next(this._messages.find(v => v.id === id)!);
            this._messages = this._messages.filter(v => v.id !== id);
            clearTimeout(timeout);
        }, 2500);

        this.added.next(msg);
    }
    public error = (message: string | Error) => {
        if (message instanceof HttpErrorResponse) this.add(message.error.error, 'error');
        else if (message instanceof Error) this.add(message.message, 'error');
        else this.add(message, 'error');
    }
    public warn = (message: string) => this.add(message, 'warning');
    public success = (message: string) => this.add(message, 'success');
    public info = (message: string) => this.add(message, 'info');
}
